cordova.define("com.holenderski.connection.AppConnection", function(require, exports, module) {
'use strict';

/*
	In order to have bi-directional traffic over a single socket, we assume:

	- each response starts with the word "status",
	- the responses are provided in the same order of the requests,
	- incoming server commands do not start with the word "status".

	These assumptions allow us to store the response handlers in a FIFO. An incoming
	server command triggers the incomingCallback.


	Alternative approach, allowing out-of-order responses (not implemented):

	In order to have bi-directional traffic over a single socket, where both endpoints
	can initiate a message exchange asynchronousely, the messages need to be tagged to 
	match the response with the request. Therefore, each message is prepended with a
	message id.

	E.g. when the client wants to send a "hello world" message to the server,
	then the client will actually send
		
		client.123 hello world\n

	With client.123 being the message identifier <source>.<sequence>, where <source>
	distinguishes betwee the innitiator of the request and <sequence> is the sequence 
	number of the message.

	The server will respond with message "hi" by sending

		client.123 hi\n
*/ 

/* global module, require, document */
var exec = require('cordova/exec');

/** @constructor */
function AppConnection (host, port) {
	//debug("AppConnection.create " + host + ":" + port);
	this.host = host;
	this.port = port;

	// function (command) {...}
	this.incomingCallback = undefined;

	// [ { success: function () {...}, failure: function (err) {...} }, ... ]
	this.openCallbacks = []; 
	this.closeCallbacks = []; 

	// { "xyz": { success: function () {...}, failure: function (err) {...} }, timer: timer }, ... }
	this.responseCallbacks = {};

	// "closed" -> "opening" -> "open"
	this.state = "closed";

	// holder for the socket id returned by the socket plugin
	this.id = host + ":" + port;

	// message sequence id
	this.messageId = 1;
}

// connections, e.g. { "host:port": connection, ... }
AppConnection.connections = {};

AppConnection.get = function (host, port) {
	if (!host) { debug("AppConnection.get: host == null"); return; }
	if (!port) { debug("AppConnection.get: port == null"); return; }

	var connectionId = host + ":" + port;

	// create the connection, if not already created
	var connection = AppConnection.connections[connectionId];		
	if (!connection) {
		//debug("AppConnection.get(): create new connection with id " + connectionId)
		connection = new AppConnection(host, port);
		connection.connectionId = connectionId;
		AppConnection.connections[connectionId] = connection;
	}

	return connection;
}

/**
 * Close all connections.
 */

AppConnection.closeAll = function () {
	for (var key in AppConnection.connections) {
		var connection = AppConnection.connections[key];
		connection.close();
	}
}

AppConnection.setValidateCertificateChain = function (validate, successCallback, failureCallback) {
	Native.setValidateCertificateChain(successCallback, failureCallback, validate);
}

AppConnection.prototype.send = function (command, successCallback, failureCallback, options) {
	options = Utils.merge({
		timeout: 0
	}, options);

	var that = this;

	var commandName = command.length > 0 ? command.split(" ")[0] : "";

	// append a message id
	this.messageId++;
	command = "call " + this.messageId + " " + command;

	// setup timeout timer, which will be cancelled in the receive handler
	var timer = undefined;
	if (options.timeout > 0) {
	  timer = setTimeout(function () {
	    debug("AppConnection.send(" + commandName + "): TIMEOUT");
	    that.onTimeout(failureCallback);
	  }, options.timeout);
	}

	// register the success callback
	this.responseCallbacks[this.messageId] = { success: successCallback, failure: failureCallback, timer: timer };
	this.sendCallbackId++;

	// escape all \n
	command = command.replace(/\n/g, "\\n");

	Native.send(
		function () {}, /* success will be communicated via onReceived() */
		function (err) {
		  debug('failed sending "' + err + '"');
		  clearTimeout(timer);
		  failureCallback && failureCallback("network_error");
		},
		this.id,
		command
	);
}

AppConnection.prototype.receive = function (message) {
	// distinguish between a responce to a previous call and an async server command
	// based on the first word in the message
	var words = message.split(" ");
	if (words.length == 0) { return; }

	// check if result, i.e. first word is "reply"
	switch (words[0]) {
		case "reply":
			// get the corresponding callback
			if (words.length < 2) { 
				alert("received a response with no message id");
				return;
			}
			var messageId = words[1];

			// extract the reply message
			message = words.slice(2).join(" ");

			var callback = this.responseCallbacks[messageId];
			if (callback) {
				// cancel the timeout timer
      	clearTimeout(callback.timer);

      	// handle the message
				callback.success && callback.success(message);
			} else {
				debug("received a response with no handler to handle it (" + message.substr(0, 200) + ")");
			}

			delete this.responseCallbacks[messageId];
			break;

		default: /* command from the server */
			if (this.incomingCallback) {
				this.incomingCallback(message);
			}
			break;
	}
}

/*
 * Execute and clear a set of callbacks. Create a sort of a barrier: execute all callbacks
 * currently pending. Do not execute any new callbacks which may have been added during
 * the execution.
 */

AppConnection.prototype.executeCallbacks = function (callbacks, action, result, message) {
	/*
	debug("executeCallbacks:\n" + Utils.dump({
			connection: this.id,
			message: message,
			callbacks: callbacks,
			action: action,
		  "this[callbacks]": this[callbacks]
	}));
	*/

	// clear the old callbacks first, in case new callbacks will be added within callbacks
	var oldCallbacks = this[callbacks];
  this[callbacks] = Array.isArray(this[callbacks]) ? [] : {};

	// use a generic iterator for arrays (e.g. openCallbacks) and objects (e.g. responseCallbacks)
	for (var id in oldCallbacks) {
		var callback = oldCallbacks[id];
		clearTimeout(callback.timer);
  	callback[action] && callback[action](result);
  }
}

AppConnection.prototype.onClosed = function (where) {
	debug("AppConnection.onClosed(): closeCallbacks=" + this.closeCallbacks.length + " where: " + where)

	this.state = "closed";
	delete this.auth;

  this.executeCallbacks("openCallbacks", "failure", "network_error", "AppConnection.onClosed(): calling open failure callback");
  this.executeCallbacks("responseCallbacks", "failure", "network_error", "AppConnection.onClosed(): calling response failure callback");
  this.executeCallbacks("closeCallbacks", "success", undefined, "AppConnection.onClosed(): calling close success callback");
}

AppConnection.prototype.onOpened = function () {
	//debug("AppConnection.onOpened(): opened connection " + this.id + " with " + this.openCallbacks.length + " callbacks");

	this.state = "open";
	this.executeCallbacks("openCallbacks", "success");
}

AppConnection.prototype.onTimeout = function (failureCallback, remover) {
	var that = this;
	debug("AppConnection.onTimeout()");

	function callback() {
		debug("TIMEOUT callback", failureCallback);
		that.onClosed("timeout");
		//failureCallback && failureCallback("network_error");
		//remover && remover();
  
  	// setup periodic polling for network to automatically reenable it
  	if (navigator.connection && !AppConnection.pollNetworkTimer) {
			AppConnection.pollNetworkTimer = setInterval(function () {
				debug("NETWORK poll")
				var networkState = navigator.connection.type;
				if (networkState != Connection.NONE) {
					clearInterval(AppConnection.pollNetworkTimer);
					AppConnection.pollNetworkTimer = undefined;

					debug("NETWORK fixing")
	    		cordova.fireDocumentEvent("online");			
				}
			}, 5000)
  	}
	}

	that.state = "closing";
	Native.close(callback, callback, that.id);
}


/*
 * Setup timeout timer, which will be cancelled in the successCallback.
 */

AppConnection.prototype.queueTimeoutInArray = function (callbacks, successCallback, failureCallback, options) {
	var that = this;

	function remover() {
		var i = Utils.indexOf(that[callbacks], "failure", failureCallback);
		that[callbacks].splice(i, 1);
	}

	var timer = undefined;
	if (options.timeout > 0) {
	  timer = setTimeout(function () {
	  	that.onTimeout(failureCallback, remover);
	  }, options.timeout);
	}

	that[callbacks].push({ success: successCallback, failure: failureCallback, timer: timer });
}

/*
 * Public methods
 */

AppConnection.prototype.open = function (successCallback, failureCallback, options) {
	options = Utils.merge({
		timeout: 0
	}, options);

	//debug("AppConnection.open(): state=" + this.state);
	var that = this;

	function openFailure(err) {
		that.onClosed("openFailure " + err);
	}

	function open() {
		that.state = "opening";
  	Native.open(function () {}, openFailure, that.host, that.port);
	}

	switch (that.state) {
		case "open":
			// return success if already open
			that.openCallbacks.push({ success: successCallback, failure: failureCallback });
			that.onOpened();
			return;

		case "opening":
			// add the handler to the opening handlers
			that.queueTimeoutInArray("openCallbacks", successCallback, failureCallback, options);
			return;

		case "closing":
			// open after it was closed, otherise opening can be regarded a success
			that.closeCallbacks.push({ success: open, failure: successCallback });
			return;			

		case "closed":
			// add the handler to the opening handlers
			that.queueTimeoutInArray("openCallbacks", successCallback, failureCallback, options);

			// open the connection
			open();
			return;
	}
}
	
AppConnection.prototype.close = function (successCallback, failureCallback, options) {
	options = Utils.merge({
		timeout: 0,
		immediate: false
	}, options);

	//debug("AppConnection.close(): state=" + this.state);
	var that = this;

	function closeFailure() {
		/* old state must have been "open" (called from case "open" or after sucessfully openning in case "opening") */
		that.state = "open";
  	that.executeCallbacks("closeCallbacks", "failure", "network_error", "AppConnection.close(): calling close failure callback")
	}

	function close() {
		that.state = "closing";
		Native.close(function () {}, closeFailure, that.id);
	}

	if (options.immediate) {
		that.closeCallbacks.push({ success: successCallback, failure: failureCallback });
		close();
		return;
	}

	switch (this.state) {
		case "open":
			// add the handler to the closing handlers
			// TODO: consider removing timeout for close, since it will invoke onClosed and therefore
			// the success callback, Also, unlike open and call, close is happening locally and no timeouts
			// should be needed.
			that.queueTimeoutInArray("closeCallbacks", successCallback, failureCallback, options);

			// close immediately
			close();
			return;

		case "opening":
			// close after it was openend, otherwise closing can be regarded a success
			that.openCallbacks.push({ success: close, failure: successCallback });
			return;

		case "closing":
			// add the handler to the closing handlers
			// TODO: consider removing timeout for close, since it will invoke onClosed and therefore the success callback
			that.queueTimeoutInArray("closeCallbacks", successCallback, failureCallback, options);
			return;

		case "closed":
			// return success if already closed
			that.closeCallbacks.push({ success: successCallback, failure: failureCallback });
			that.onClosed("connection close");
			return;
	}
}

AppConnection.prototype.registerOnClosed = function (callback) {
	// add the handler to the closing handlers
	this.closeCallbacks.push({ success: callback, failure: callback });
}

AppConnection.prototype.call = function(command, successCallback, failureCallback, options) {
	options = Utils.merge({
		timeout: 0
	}, options);

	//debug("AppConnection.call(): state=" + this.state);
	var that = this;

	function openSuccess() {
		that.send(command, successCallback, failureCallback, options);
	}

	function closeSuccess() {
		that.open(openSuccess, failureCallback, options);
	}

	switch (that.state) {
		case "open":
			// send the command immediately
			openSuccess();
			return;

		case "opening":
			// send the command after opening the connection
			that.openCallbacks.push({ success: openSuccess, failure: failureCallback });
			return;

		case "closing":
			// send the command after closing and then opening the connection
			that.closeCallbacks.push({ success: closeSuccess, failure: failureCallback });
			return;

		case "closed":
			// open the connection and then send the command
			closeSuccess()
			return;
	}
}

/*
 * Hooks for the native part.
 */

//
AppConnection.onReceived = function (connectionId, message) {
   message = Utils.b64_to_utf8(message);

  // pass the message to the correct connection
  // Note: assume here that the connectionId == connection.id
	var connection = AppConnection.connections[connectionId];
	if (connection) {
		connection.receive(message);
	} else {
		debug("WARNING: AppConnection: received message without connection '" + message + "'")
	}
};

AppConnection.onClosed = function (connectionId, reason) {
	// reason examples:
	//   - CFNetwork SSLHandshake failed (-9807)
	debug("AppConnection.onClosed(): reason: " + reason);

  // pass the message to the correct connection
  // Note: assume here that the connectionId == connection.id
	var connection = AppConnection.connections[connectionId];
	if (connection) {
		connection.onClosed("hook");
	} else {
		debug("AppConnection.onClosed(): connection missing")
	}
}

AppConnection.onOpened =	function (connectionId) {
	//debug("AppConnection.onOpened(): " + connectionId);

	var connection = AppConnection.connections[connectionId];
	if (connection) {
		connection.onOpened();
 	} else {
		debug("AppConnection.onOpened(): connection missing")
	}
}

AppConnection.onOpenFailed =	function (connectionId) {
	var connection = AppConnection.connections[connectionId];
	if (connection) {
		connection.onClosed("onOpenFailed");
	} else {
		debug("AppConnection.onOpenFailed(): connection missing")
	}
}

/*
 * Native interface for AppConnection
 */

function Native() {}

// delays for calls to the native part, for testing
AppConnection.testOpenDelay = 0;
AppConnection.testCallDelay = 0;
AppConnection.testCloseDelay = 0;

Native.PluginName = "ConnectionPlugin";

//
Native.open = function (successCallback, errorCallback, host, port) {
	setTimeout(function () {
		exec(successCallback, errorCallback, Native.PluginName, 'open', [host, port]);
	}, AppConnection.testOpenDelay);
};

//
Native.send = function (successCallback, errorCallback, connectionId, data) {
	setTimeout(function () {
		exec(successCallback, errorCallback, Native.PluginName, 'send', [connectionId, typeof data == 'string' ? data : JSON.stringify(data)]);
	}, AppConnection.testCallDelay);
};

//
Native.close = function (successCallback, errorCallback, connectionId) {
	setTimeout(function () {
		exec(successCallback, errorCallback, Native.PluginName , 'close', [connectionId]);
	}, AppConnection.testCloseDelay);
};

//
Native.setValidateCertificateChain = function (successCallback, errorCallback, validate) {
	setTimeout(function () {
		exec(successCallback, errorCallback, Native.PluginName , 'setValidateCertificateChain', [validate]);
	}, AppConnection.testCloseDelay);
};

module.exports = AppConnection;


});
